require(tidyverse)
require(readxl)

# functions ---------------------------------------------------------------

extract_time <- function(x) {
  format <- "%H:%M:%S"
  as.POSIXct(strftime(x, format=format), format=format)
}

as_yearly_period <- function(x) {
  x <- (as.numeric(x) - as.numeric(as.Date("2010-01-01"))) / 365 * 2 * pi
}


# constants ---------------------------------------------------------------

start_date <- '2013-09-01'
datetime_format <- '%d.%m.%Y %H:%M:%S'
weekday_names <- c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")

WORKING_DAY <- 'Working days w/o Friday'
FRIDAY <- 'Friday'
WEEKEND <- 'Weekend / Holiday'
DAY_CAT_LEVELS <- c(WORKING_DAY, FRIDAY, WEEKEND)


# loading -----------------------------------------------------------------

calls <- read_csv('data/Calls.csv', col_types = list(
  DATETIME = col_datetime(format = datetime_format),
  HOTLINE_START_TIME = col_datetime(format = datetime_format),
  HOTLINE_END_TIME = col_datetime(format = datetime_format),
  ROK_HLASENI = col_integer(),
  CISLO_HLASENI = col_integer()
)) %>% 
  set_names(tolower) %>% 
  mutate(time = as.POSIXct(strftime(datetime, format="%H:%M:%S"), format="%H:%M:%S"),
         duration = difftime(hotline_end_time, hotline_start_time, units = 'mins')) %>% 
  filter(datetime >= start_date) %>% 
  select(-rok_hlaseni, -cislo_hlaseni)

releases <- read_xlsx('data/distribution dates.xlsx', range = 'E2:F76', col_names =c('date', 'is_release')) %>% 
  mutate(date = as.Date(date))
holidays <- read_excel('data/svatky.xlsx', col_names = c('date')) %>% 
  mutate(is_holiday = 1, date = as.Date(date))


# transformation / aggregation --------------------------------------------

daily_calls <- calls %>% 
  group_by(date = as.Date(datetime)) %>% 
  count(name = 'num_calls') %>% 
  ungroup() %>% 
  complete(date = seq.Date(as.Date(start_date), max(date), by="day"), fill = list(num_calls = 0L)) %>% 
  mutate(weekday = factor(weekdays(date), levels = weekday_names), 
         month = months(date),
         yearly_period = as_yearly_period(date)) %>% 
  left_join(releases, by = 'date') %>% 
  left_join(holidays, by = 'date') %>% 
  mutate(
    is_release = ifelse(is.na(is_release), 0L, 1L),
    is_holiday = ifelse(is.na(is_holiday), 0L, 1L),
    days_since_release = accumulate(is_release, ~ifelse(.y == 1, 0, .x + 1), .init = NA)[-1],
    days_since_release_cap = pmin(days_since_release, 20),
    release_decay = accumulate(is_release, ~ifelse(.y == 1, 1, .x * 0.9)),
    day_cat = case_when(
      is_holiday == 1 ~ WEEKEND,
      as.integer(weekday) %in% 6:7 ~ WEEKEND,
      as.integer(weekday) == 5 ~ FRIDAY,
      TRUE ~ WORKING_DAY
    ),
    day_cat = factor(day_cat, levels = DAY_CAT_LEVELS)
  ) %>% 
  filter(!is.na(days_since_release))

calls %>% 
  filter(!is.na(hotline_start_time), !is.na(hotline_end_time)) %>% 
  write_csv('processed/calls.csv')
write_csv(daily_calls, 'processed/daily_calls.csv')
